#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#define DEBUG 0

int main (int argc,  char *argv[])
{
	char processLine[6144], atom[5], *atomValue, sourceLine[4096], *atomName;
	char single[100], apPath[100] = "", directory[100], newFile[100], shortNewFile[100], cleanFileName[100];
	char output[100] = "./results";
	char blank[100] = "./blankvideo.mp4";
	FILE *fd;
	int x, y, z, fileCount = 0, singleB = 0, blankB = 0, apPathB = 0, directoryB = 0, outputB = 0;
	struct entry {
		char iTunMOVI[4096], ldes[1248];
		int covr;
		char Cnam[128], desc[512], Cart[128];
		char iTunEXTC[128], Cday[128], cpil[128], pcst[128], stik[128], Cgen[128], rtng[128], pgap[128], AltT[128], fileName[128];
		struct entry *next;
	};
	struct entry *entryCurrent = NULL, *entryList = NULL;

//////////////////////////////////////////////////////////////////////
//   Arguement Management                                           //
//////////////////////////////////////////////////////////////////////

	if ((argc>1) && (DEBUG)) printf("Arguement: %s\n",argv[1]);
	if (argc==1) {
		fprintf(stderr,"Error:  No options entered.  Please refer to help using '-h' option.\n");
		return (1);
	}
	if ((strcmp(argv[1], "-h")==0) || (strcmp(argv[1], "--help"))==0) {
		fprintf(stderr,"\n\nalm-script1 (ATV Link Maker - Script 1) - Version 0.80\n");
		fprintf(stderr,"https://bitbucket.org/TexasEric/alm-script/overview\n\n");
		fprintf(stderr,"ASSUMPTIONS:\n");
		fprintf(stderr,"This program assumes you have installed AtomicParsley on your system.  You must have write permissions to the desitnation directory.  This program only works with .mp4 or .m4v files.\n\n");
		fprintf(stderr,"Usage: alm-scrpit1 OPTION...\n");
		fprintf(stderr,"  -h, --help                       Present this help information\n");
		fprintf(stderr,"  -f, --file <filename>            process a single file with the given name\n                                   and path from 'pwd'\n");
		fprintf(stderr,"  -d, --directory <dir-name>       process all mp4 and m4v files inside the\n                                   given directory.  Don't put spaces in file\n                                   name\n");
		fprintf(stderr,"  -b, --blank <blank file>         alternate location AND name for blank file\n                                   template. Default is ./blankvideo.mp4\n");
		fprintf(stderr,"  -o, --output <output folder>     alternate location for output files. Folder\n                                   will be made if it does not exist. Default\n                                   is ./results\n");
		fprintf(stderr,"  -a, --appath <path to AtomicParsley>\n                                   alternate location for AtomicParsely\n                                   executable. Default is somewhere in $PATH\n");
		fprintf(stderr,"\n\nExamples: ./alm-script1 --appath . --directory /mnt/Media/Movies\n");
		fprintf(stderr,"          ./alm-script1 -f /mnt/Media/Movies/themovie.m4v\n\n");
		fprintf(stderr,"This program may throw various errors while it is running.  The following are acceptable and no cause for concern:\n");
		fprintf(stderr,"  - references to 'mv' not finding a file immediately following a 'No changes.'\n    line likely indicates that some atoms may have not been defined in the\n    original file and will not exist when writing.  This is typically\n    acceptable.\n");
		return(0);
	}

	for (x=1; x<argc; x++)
	{
		if (DEBUG) fprintf(stderr,"Arguement %i: %s\n",x,argv[x]);
		if ((strcmp(argv[x], "-f")==0) || (strcmp(argv[x], "--file")==0)) {
			singleB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') single[z++]=argv[x+1][y];
				else {
					single[z++]='\\';
					single[z++]=' ';
				}
			single[z]='\0';
		}
		if ((strcmp(argv[x], "-d")==0) || (strcmp(argv[x], "--directory")==0)) {
			directoryB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') directory[z++]=argv[x+1][y];
				else {
					directory[z++]='\\';
					directory[z++]=' ';
				}
			directory[z++]='/';
			directory[z]='\0';
		}
		if ((strcmp(argv[x], "-o")==0) || (strcmp(argv[x], "--output")==0)) {
			outputB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') output[z++]=argv[x+1][y];
				else {
					output[z++]='\\';
					output[z++]=' ';
				}
			output[z]='\0';
		}
		if ((strcmp(argv[x], "-b")==0) || (strcmp(argv[x], "--blank")==0)) {
			blankB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') blank[z++]=argv[x+1][y];
				else {
					blank[z++]='\\';
					blank[z++]=' ';
				}
			blank[z]='\0';
			if((fd = fopen(blank, "rb"))==NULL) {
				printf("Error: Cannot open blank template file [%s].\n",blank);
    				exit(1);
			}
			fclose(fd);
		}
		if ((strcmp(argv[x], "-a")==0) || (strcmp(argv[x], "--appath")==0)) {
			apPathB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') apPath[z++]=argv[x+1][y];
				else {
					apPath[z++]='\\';
					apPath[z++]=' ';
				}
			apPath[z++]='/';
			apPath[z]='\0';
		}
	}
	if ((singleB + directoryB)!=1) {
		fprintf(stderr,"Error: Source not properly determined.\n");
		return(1);
	}

//////////////////////////////////////////////////////////////////////
//   Pre-processing file management                                 //
//////////////////////////////////////////////////////////////////////

	// make output directory (if it does not exist)
	if (DEBUG) fprintf(stderr, "Making %s\n", output);
	mkdir (output, S_IRWXG | S_IRWXO | S_IRWXU);

	// remove old processing files (if they exist)
	sprintf(processLine,"%s/apDataTemp",output);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);
	sprintf(processLine,"%s/md5_data",output);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);

	//Get atoms using AtomicParsley
	if (singleB == 1) {
		//remove any leading path information for result folder processing
		if (strrchr(single,'/')!=NULL) strcpy(shortNewFile,strrchr(single,'/')+1);
		else strcpy(shortNewFile,single);
		sprintf(processLine, "echo 'Atom \"FILE\" contains: %s' >> %s/apDataTemp",shortNewFile,output);
		system (processLine);

		//Get data
		sprintf(processLine, "%sAtomicParsley %s -t >> %s/apDataTemp",apPath,single,output);
		system (processLine);

		//Get coverart using AtomicParsley
		sprintf(processLine, "%sAtomicParsley %s -e %s/%s",apPath,single,output,shortNewFile);
		system (processLine);
	}
	if (directoryB == 1) {
		//Get atoms using AtomicParsley
		sprintf(processLine, "ls -1 %s | grep mp4 | grep -v ._ > %s/filelist ; ls -1 %s | grep m4v | grep -v ._ >> %s/filelist",directory,output,directory,output);
		system (processLine);

		sprintf(processLine,"%s/filelist",output);
		fd = fopen(processLine,"r");
		while (fgets(sourceLine,100,fd) != NULL) {
			y = 0;
			z = 0;
			//if file name has a space in it, put a \ in front.
			for (y=0;y<strlen (sourceLine);y++) {
				if ((sourceLine[y]== ' ') ||(sourceLine[y]== '&')) newFile[z++]='\\';
				newFile[z++] = sourceLine[y];
			}
			newFile[z-1]='\0';
			//remove any leading path information for result folder processing
			if (strrchr(newFile,'/')!=NULL) strcpy(shortNewFile,strrchr(newFile,'/')+1);
			else strcpy(shortNewFile,newFile);

			sprintf(processLine, "echo 'Atom \"FILE\" contains: %s' >> %s/apDataTemp",shortNewFile,output);
			system (processLine);

			//Save proper file name for title in case cnam tag does not exist.
			strcpy(cleanFileName,sourceLine);
			if (strrchr(cleanFileName,'/')!=NULL) strcpy(cleanFileName,strrchr(cleanFileName,'/')+1);
			cleanFileName[strlen(cleanFileName)-5] = '\0';
			sprintf(processLine, "echo 'Atom \"AltT\" contains: %s' >> %s/apDataTemp",cleanFileName,output);
			system (processLine);

			sprintf(processLine, "%sAtomicParsley %s%s -t >> %s/apDataTemp",apPath,directory,newFile,output);
			system (processLine);


			//Get coverart using AtomicParsley
			sprintf(processLine, "%sAtomicParsley %s%s -e %s/%s",apPath,directory,newFile,output,shortNewFile);
			system (processLine);

		}
		fclose (fd);
	}

	// Data file in place, build fopen string
	sprintf(processLine,"%s/apDataTemp",output);
	if (DEBUG) fprintf(stderr, "Ready to read data into memory: %s\n", processLine);

//////////////////////////////////////////////////////////////////////
//   Read Input Into List                                           //
//////////////////////////////////////////////////////////////////////

	fd = fopen(processLine,"r");

	while (fgets(sourceLine,4096,fd) != NULL) {
		atomName = strstr(sourceLine, "Atom");
		if (atomName) {
			atomValue = strstr(sourceLine, "contains:")+10;
			memset (atom,'\0',sizeof (atom));
			strncpy(atom,atomName+6,4);
			if ((atom[0]==-62) && (atom[1]==-87)) { //some atoms start with what looks like a copyright symbol.  Lets fix that.
				strncpy(atom,atomName+7,4);
				atom[0]='c';
			}
			//Chop off extra characters at end of line.  If its the self-injected file name, only cut off the 1 newline char.
			sourceLine[strlen(sourceLine)-1] = '\0';
			for (x=0;x<strlen(atomValue);x++)
				if (atomValue[x]=='\"') atomValue[x]='\'';
			if (strncmp(atom,"FILE",4)==0) {
				// cnam atom means we have a new entry.  cnam is always the first atom produced from AtomicParsley.
				if (entryCurrent != NULL) {
					entryCurrent->next = (struct entry *) malloc(sizeof (struct entry));
					entryCurrent = entryCurrent->next;
				}
				else entryCurrent = (struct entry *) malloc(sizeof (struct entry));
				if (entryList == NULL) entryList = entryCurrent;
				entryCurrent->next = NULL;

				// fill fileName atom value and a couple of default values				
				strncpy(entryCurrent->fileName,atomValue,128);	
				strcpy(entryCurrent->pgap,"false");
				entryCurrent->Cnam[0] = '\0';

				fileCount = fileCount + 1;
				if (DEBUG) fprintf (stderr,"File Name: %s\n",entryCurrent->fileName);
			}
			else if (strcmp(atom,"ldes")==0) strncpy(entryCurrent->ldes,atomValue,1248);
			else if (strcmp(atom,"desc")==0) strncpy(entryCurrent->desc,atomValue,512);
			else if (strcmp(atom,"cnam")==0) strncpy(entryCurrent->Cnam,atomValue,128);				
			else if (strcmp(atom,"cART")==0) strncpy(entryCurrent->Cart,atomValue,128);
			else if (strcmp(atom,"cday")==0) strncpy(entryCurrent->Cday,atomValue,128);
			else if (strcmp(atom,"cpil")==0) strncpy(entryCurrent->cpil,atomValue,128);
			else if (strcmp(atom,"pcst")==0) strncpy(entryCurrent->pcst,atomValue,128);
			else if (strcmp(atom,"stik")==0) strncpy(entryCurrent->stik,atomValue,128);
			else if (strcmp(atom,"cgen")==0) strncpy(entryCurrent->Cgen,atomValue,128);
			else if (strcmp(atom,"rtng")==0) strncpy(entryCurrent->rtng,atomValue,128);
			else if (strcmp(atom,"AltT")==0) strncpy(entryCurrent->AltT,atomValue,128);
			else if (strcmp(atom,"pgap")==0) strncpy(entryCurrent->pgap,atomValue,128);
			else if (strcmp(atom,"covr")==0) entryCurrent->covr = atoi(atomValue);
			else if (strcmp(atom,"----")==0) {
				if(strstr(sourceLine, "iTunEXTC")) strncpy(entryCurrent->iTunEXTC,atomValue,128);
				if(strstr(sourceLine, "iTunMOVI")) {
					strncpy(entryCurrent->iTunMOVI,atomValue,4096);
					y=strlen(entryCurrent->iTunMOVI);
					if (DEBUG) fprintf(stderr,"First iTunMOVI: [%i][%s]\n",y,entryCurrent->iTunMOVI);
					if (strstr(entryCurrent->iTunMOVI, "/plist") == NULL) {
						do {					
							fgets(sourceLine,4096,fd);
							for (x=0;x<strlen(sourceLine); x++)
								if ((sourceLine[x]>=32) && (sourceLine[x]<= 126) && (y<4096)) entryCurrent->iTunMOVI[y++]=sourceLine[x];
						} while ((strstr(sourceLine, "/plist") == NULL));
						entryCurrent->iTunMOVI[y]='\0';
					}
					for (x=0;x<strlen(entryCurrent->iTunMOVI); x++)
						if (entryCurrent->iTunMOVI[x]=='\'') entryCurrent->iTunMOVI[x]='\"';
					if (DEBUG) fprintf(stderr,"iTunMOVI: [%s][%s]\n",entryCurrent->iTunMOVI, sourceLine);
				}
			}
		}
	if (DEBUG) fprintf(stderr, "Just read: %s\n", atom);
	} // while statement to read data
	fprintf (stderr, "\n\nFiles identified for processing: %i.\n\n",fileCount);
	fclose(fd);

//////////////////////////////////////////////////////////////////////
//   Debug Output to Screen                                         //
//////////////////////////////////////////////////////////////////////

	entryCurrent = entryList;
	if (DEBUG) while (entryCurrent != NULL) {
		fprintf (stderr,"Name: %s\n",entryCurrent->Cnam);
		fprintf (stderr,"   FILE NAME: %s\n",entryCurrent->fileName);
		fprintf (stderr,"   cART: %s\n",entryCurrent->Cart);
		fprintf (stderr,"   ldes: %s\n",entryCurrent->ldes);
		fprintf (stderr,"   cday: %s\n",entryCurrent->Cday);
		fprintf (stderr,"   iTunEXTC: %s\n",entryCurrent->iTunEXTC);
		fprintf (stderr,"   iTunMOVI: %s\n",entryCurrent->iTunMOVI);
		fprintf (stderr,"   cpil: %s\n",entryCurrent->cpil);
		fprintf (stderr,"   pcst: %s\n",entryCurrent->pcst);
		fprintf (stderr,"   stik: %s\n",entryCurrent->stik);
		fprintf (stderr,"   cgen: %s\n",entryCurrent->Cgen);
		fprintf (stderr,"   rtng: %s\n",entryCurrent->rtng);
		fprintf (stderr,"   pgap: %s\n",entryCurrent->pgap);
		fprintf (stderr,"   covr: %i\n",entryCurrent->covr);

		entryCurrent = entryCurrent->next;
	} 

//////////////////////////////////////////////////////////////////////
//   Make Files!!                                                   //
//////////////////////////////////////////////////////////////////////

	fprintf (stderr, "\n\nMaking Files\n\n");
	entryCurrent = entryList;
	fileCount = 0;
	while (entryCurrent != NULL) {
		if((fd = fopen(blank, "rb"))==NULL) {
			printf("Error: Cannot open blank template file.\n");
    			exit(1);
		}
		fclose(fd);
		sprintf(processLine, "cp %s %s/%s",blank,output,entryCurrent->fileName);
		fprintf(stderr,"\n\n----------------------------------------------------------\nMaking new template file: %s\n----------------------------------------------------------\n",entryCurrent->fileName);
		system(processLine);
		
		// For whatever reason, AtomicParsley does not reliably --overWrite or --output.  Let it make temps and we will just move the resulting files.
		strcpy(newFile,entryCurrent->fileName);
		newFile[strlen(newFile)-4]='\0';
		strcat(newFile,"-temp-");
		
		//If Title Atom (cnam) is empty, fill it with File Name without file extension
		if (strlen(entryCurrent->Cnam)==0) {
			strcpy(entryCurrent->Cnam,entryCurrent->AltT);
			fprintf (stderr, "Warning: %s does not have a title atom.  Using File Name: %s\n",entryCurrent->fileName,entryCurrent->Cnam);
		}
		sprintf(processLine,"%sAtomicParsley %s/%s --title \"%s\" --artist \"%s\"",apPath,output,entryCurrent->fileName,entryCurrent->Cnam,entryCurrent->Cart);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		sprintf(processLine,"%sAtomicParsley %s/%s --compilation \"%s\" --podcastFlag \"%s\"",apPath,output,entryCurrent->fileName,entryCurrent->cpil,entryCurrent->pcst);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		sprintf(processLine,"%sAtomicParsley %s/%s --gapless %s --longdesc \"%s\"",apPath,output,entryCurrent->fileName,entryCurrent->pgap,entryCurrent->ldes);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		sprintf(processLine,"%sAtomicParsley %s/%s --stik \"%s\" --genre \"%s\"",apPath,output,entryCurrent->fileName,entryCurrent->stik,entryCurrent->Cgen);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		sprintf(processLine,"%sAtomicParsley %s/%s --rDNSatom '%s' name=iTunMOVI domain=com.apple.itunes",apPath,output,entryCurrent->fileName,entryCurrent->iTunMOVI);
		system(processLine);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		sprintf(processLine,"%sAtomicParsley %s/%s --description \"%s\" --year \"%s\"",apPath,output,entryCurrent->fileName,entryCurrent->desc,entryCurrent->Cday);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		sprintf(processLine,"%sAtomicParsley %s/%s --rDNSatom \"%s\" name=iTunEXTC domain=com.apple.itunes",apPath,output,entryCurrent->fileName,entryCurrent->iTunEXTC);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);
		sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
		if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
		system(processLine);

		if (entryCurrent->covr > 0) {
			sprintf(processLine,"%sAtomicParsley %s/%s --artwork %s/%s_artwork_1.*",apPath,output,entryCurrent->fileName,output,entryCurrent->fileName);
			if (DEBUG) fprintf(stderr,"cover: %s\n",processLine);
			system(processLine);
			sprintf(processLine, "mv %s/%s* %s/%s",output,newFile,output,entryCurrent->fileName);
			if (DEBUG) fprintf(stderr,"Processing: %s\n",processLine);
			system(processLine);
		}
		else fprintf (stderr," No cover art found for: %s\n",entryCurrent->fileName);

		//Generate MD5 data for script2
		sprintf(processLine,"md5 %s/%s >> %s/md5_data",output, entryCurrent->fileName, output);
		system(processLine);

		if (DEBUG) fprintf(stderr,"File processed...  Any more?\n");
		fileCount++;
		entryCurrent = entryCurrent->next;
	}

//////////////////////////////////////////////////////////////////////
//   Post-processing clean-up                                       //
//////////////////////////////////////////////////////////////////////

	// remove old processing files (if they does exist)
	sprintf(processLine,"%s/apDataTemp",output);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);
	sprintf(processLine,"%s/filelist",output);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);

	fprintf (stderr,"\n\n----------------------------------------------------------\nAll Done.  %i files created in the %s directory.\nAdd these file to iTunes,'sync' them to your Apple TV, and run Script 2 on the ATV.\n\n",fileCount,output);
}
